"use strict";

const conversions = require("webidl-conversions");
const utils = require("./utils.js");
const HTMLElement = require("./HTMLElement.js");
const impl = utils.implSymbol;

function HTMLMeterElement() {
  throw new TypeError("Illegal constructor");
}
HTMLMeterElement.prototype = Object.create(HTMLElement.interface.prototype);
HTMLMeterElement.prototype.constructor = HTMLMeterElement;


HTMLMeterElement.prototype.toString = function () {
  if (this === HTMLMeterElement.prototype) {
    return "[object HTMLMeterElementPrototype]";
  }
  return HTMLElement.interface.prototype.toString.call(this);
};
Object.defineProperty(HTMLMeterElement.prototype, "value", {
  get() {
    return utils.tryWrapperForImpl(this[impl].value);
  },
  set(V) {
    V = conversions["double"](V);
    this[impl].value = utils.tryImplForWrapper(V);
  },
  enumerable: true,
  configurable: true
});

Object.defineProperty(HTMLMeterElement.prototype, "min", {
  get() {
    return utils.tryWrapperForImpl(this[impl].min);
  },
  set(V) {
    V = conversions["double"](V);
    this[impl].min = utils.tryImplForWrapper(V);
  },
  enumerable: true,
  configurable: true
});

Object.defineProperty(HTMLMeterElement.prototype, "max", {
  get() {
    return utils.tryWrapperForImpl(this[impl].max);
  },
  set(V) {
    V = conversions["double"](V);
    this[impl].max = utils.tryImplForWrapper(V);
  },
  enumerable: true,
  configurable: true
});

Object.defineProperty(HTMLMeterElement.prototype, "low", {
  get() {
    return utils.tryWrapperForImpl(this[impl].low);
  },
  set(V) {
    V = conversions["double"](V);
    this[impl].low = utils.tryImplForWrapper(V);
  },
  enumerable: true,
  configurable: true
});

Object.defineProperty(HTMLMeterElement.prototype, "high", {
  get() {
    return utils.tryWrapperForImpl(this[impl].high);
  },
  set(V) {
    V = conversions["double"](V);
    this[impl].high = utils.tryImplForWrapper(V);
  },
  enumerable: true,
  configurable: true
});

Object.defineProperty(HTMLMeterElement.prototype, "optimum", {
  get() {
    return utils.tryWrapperForImpl(this[impl].optimum);
  },
  set(V) {
    V = conversions["double"](V);
    this[impl].optimum = utils.tryImplForWrapper(V);
  },
  enumerable: true,
  configurable: true
});

Object.defineProperty(HTMLMeterElement.prototype, "labels", {
  get() {
    return utils.tryWrapperForImpl(this[impl].labels);
  },
  enumerable: true,
  configurable: true
});


module.exports = {
  mixedInto: [],
  is(obj) {
    if (obj) {
      if (obj[impl] instanceof Impl.implementation) {
        return true;
      }
      for (let i = 0; i < module.exports.mixedInto.length; ++i) {
        if (obj instanceof module.exports.mixedInto[i]) {
          return true;
        }
      }
    }
    return false;
  },
  isImpl(obj) {
    if (obj) {
      if (obj instanceof Impl.implementation) {
        return true;
      }

      const wrapper = utils.wrapperForImpl(obj);
      for (let i = 0; i < module.exports.mixedInto.length; ++i) {
        if (wrapper instanceof module.exports.mixedInto[i]) {
          return true;
        }
      }
    }
    return false;
  },
  create(constructorArgs, privateData) {
    let obj = Object.create(HTMLMeterElement.prototype);
    this.setup(obj, constructorArgs, privateData);
    return obj;
  },
  createImpl(constructorArgs, privateData) {
    let obj = Object.create(HTMLMeterElement.prototype);
    this.setup(obj, constructorArgs, privateData);
    return utils.implForWrapper(obj);
  },
  _internalSetup(obj) {
    HTMLElement._internalSetup(obj);

  },
  setup(obj, constructorArgs, privateData) {
    if (!privateData) privateData = {};
    privateData.wrapper = obj;

    this._internalSetup(obj);

    obj[impl] = new Impl.implementation(constructorArgs, privateData);
    obj[impl][utils.wrapperSymbol] = obj;
  },
  interface: HTMLMeterElement,
  expose: {
    Window: { HTMLMeterElement: HTMLMeterElement }
  }
};


const Impl = require("../nodes/HTMLMeterElement-impl.js");
