"use strict";

const conversions = require("webidl-conversions");
const utils = require("./utils.js");
const HTMLElement = require("./HTMLElement.js");
const impl = utils.implSymbol;

function HTMLProgressElement() {
  throw new TypeError("Illegal constructor");
}
HTMLProgressElement.prototype = Object.create(HTMLElement.interface.prototype);
HTMLProgressElement.prototype.constructor = HTMLProgressElement;


HTMLProgressElement.prototype.toString = function () {
  if (this === HTMLProgressElement.prototype) {
    return "[object HTMLProgressElementPrototype]";
  }
  return HTMLElement.interface.prototype.toString.call(this);
};
Object.defineProperty(HTMLProgressElement.prototype, "value", {
  get() {
    return utils.tryWrapperForImpl(this[impl].value);
  },
  set(V) {
    V = conversions["double"](V);
    this[impl].value = utils.tryImplForWrapper(V);
  },
  enumerable: true,
  configurable: true
});

Object.defineProperty(HTMLProgressElement.prototype, "max", {
  get() {
    return utils.tryWrapperForImpl(this[impl].max);
  },
  set(V) {
    V = conversions["double"](V);
    this[impl].max = utils.tryImplForWrapper(V);
  },
  enumerable: true,
  configurable: true
});

Object.defineProperty(HTMLProgressElement.prototype, "position", {
  get() {
    return utils.tryWrapperForImpl(this[impl].position);
  },
  enumerable: true,
  configurable: true
});

Object.defineProperty(HTMLProgressElement.prototype, "labels", {
  get() {
    return utils.tryWrapperForImpl(this[impl].labels);
  },
  enumerable: true,
  configurable: true
});


module.exports = {
  mixedInto: [],
  is(obj) {
    if (obj) {
      if (obj[impl] instanceof Impl.implementation) {
        return true;
      }
      for (let i = 0; i < module.exports.mixedInto.length; ++i) {
        if (obj instanceof module.exports.mixedInto[i]) {
          return true;
        }
      }
    }
    return false;
  },
  isImpl(obj) {
    if (obj) {
      if (obj instanceof Impl.implementation) {
        return true;
      }

      const wrapper = utils.wrapperForImpl(obj);
      for (let i = 0; i < module.exports.mixedInto.length; ++i) {
        if (wrapper instanceof module.exports.mixedInto[i]) {
          return true;
        }
      }
    }
    return false;
  },
  create(constructorArgs, privateData) {
    let obj = Object.create(HTMLProgressElement.prototype);
    this.setup(obj, constructorArgs, privateData);
    return obj;
  },
  createImpl(constructorArgs, privateData) {
    let obj = Object.create(HTMLProgressElement.prototype);
    this.setup(obj, constructorArgs, privateData);
    return utils.implForWrapper(obj);
  },
  _internalSetup(obj) {
    HTMLElement._internalSetup(obj);

  },
  setup(obj, constructorArgs, privateData) {
    if (!privateData) privateData = {};
    privateData.wrapper = obj;

    this._internalSetup(obj);

    obj[impl] = new Impl.implementation(constructorArgs, privateData);
    obj[impl][utils.wrapperSymbol] = obj;
  },
  interface: HTMLProgressElement,
  expose: {
    Window: { HTMLProgressElement: HTMLProgressElement }
  }
};


const Impl = require("../nodes/HTMLProgressElement-impl.js");
